using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;
using NiVE2.Plugin.Interface;
using NiVE2.Plugin.Property;
using NiVE2.Plugin.Controls;
using NiVE2.Drawing;
using NiVE2.Utils;
using NiVE2.Utils.Cache;
using ruche.aviutl.plugins;

namespace ruche.nive2.aviutlPluginPorts
{
    /// <summary>
    /// AviUtl̓vOC𗘗pNiVErfI̓vOCNXB
    /// MediaInput ō쐬B
    /// </summary>
    internal class VideoMediaInput : InputChild
    {
        /// <summary>
        /// eCX^XB
        /// </summary>
        private InputBase _parent = null;

        /// <summary>
        /// ̓vOCC^tF[XB
        /// </summary>
        private InputPlugin _plugin = null;

        /// <summary>
        /// ʔԍB
        /// </summary>
        private int _childCode = 0;

        /// <summary>
        /// ĐԁB
        /// </summary>
        private double _length = 0.0;

        /// <summary>
        /// t[[gB
        /// </summary>
        private double _frameRate = 0.0;

        /// <summary>
        /// RXgN^B
        /// </summary>
        /// <param name="parent">eCX^XB</param>
        /// <param name="plugin">
        /// fBAt@Cǂݍݍς݂̓̓vOCB
        /// </param>
        /// <param name="id">q̎IDB</param>
        public VideoMediaInput(
            InputBase parent, InputPlugin plugin, int id)
        {
            if (parent == null)
            {
                throw new NullReferenceException("parent  null łB");
            }
            if (plugin == null)
            {
                throw new NullReferenceException("plugin  null łB");
            }
            if (!plugin.IsMediaFileOpened)
            {
                throw new ArgumentException(
                    "fBAt@CJĂ܂B", "plugin");
            }

            // p[^ێ
            _parent = parent;
            _plugin = plugin;
            _childCode = id;

            // vl̕ێ
            IInputInfo info = _plugin.MediaFileInfo;
            _frameRate =
                (double)info.InputImageFrameRateBase / info.InputImageFrameRateScale;
            _length = info.InputImageFrameCount / _frameRate;
        }

        /// <summary>
        /// ͂̊JnsB
        /// </summary>
        void BeginInput()
        {
            // vOĈfBNgֈړ
            CurrentDirectoryStack.Instance.Push(
                Path.GetDirectoryName(_plugin.PluginFilePath));
        }

        /// <summary>
        /// ͂̏IsB
        /// </summary>
        void EndInput()
        {
            // JgfBNg
            CurrentDirectoryStack.Instance.Pop();
        }

        #region InputChild o

        public override int ChildCode
        {
            get { return _childCode; }
        }

        public override string MediaName
        {
            get
            {
                return Path.GetFileName(FileName) + "(video)";
            }
        }

        public override bool HaveVideo
        {
            get { return true; }
        }

        public override bool HaveAudio
        {
            get { return false; }
        }

        public override bool IsImage
        {
            get { return false; }
        }

        public override double Length
        {
            get { return _length; }
        }

        public override InputBase Parent
        {
            get { return _parent; }
        }

        public override MediaInfo GetMedaiInfo()
        {
            IInputInfo info = _plugin.MediaFileInfo;

            return new MediaInfo(
                _length,
                _frameRate,
                true,
                false,
                false,
                new Size(info.InputImageWidth, info.InputImageHeight));
        }

        public override PropertyBase[] GetDefaultProperty()
        {
            return new PropertyBase[]
                {
                    new SelectableProperty(
                        "YUY2XP[",
                        1,
                        new string[] {
                            "tW(Y,U,V=0`255)",
                            "kW(Y=16`235; U,V=16`240)" }),
                    new SelectableProperty(
                        "YUY2F",
                        0,
                        new string[] {
                            "(cŔ)",
                            "BT.601(SD掿)",
                            "BT.709(HD掿)" }),
                };
        }

        public override PropertyEditControlBase[] GetControl()
        {
            // YUY2`łȂȂvpeBҏW
            if (_plugin == null || !_plugin.IsMediaFileYuy2)
            {
                return new PropertyEditControlBase[] { };
            }

            return new PropertyEditControlBase[]
                {
                    new SelectablePropertyEditControl("YUY2XP["),
                    new SelectablePropertyEditControl("YUY2F"),
                };
        }

        public override RectangleF GetBounds(
            double time, IComposition composition, ReadOnlyDictionary<string, PropertyBase> property)
        {
            IInputInfo info = _plugin.MediaFileInfo;

            return new RectangleF(
                0.0F,
                0.0F,
                (float)(info.InputImageWidth * composition.ResolutionRate),
                (float)(info.InputImageHeight * composition.ResolutionRate));
        }

        public override NBitmap GetImage(
            double time, IComposition composition, ReadOnlyDictionary<string, PropertyBase> property)
        {
            NBitmap result = null;

            // vpeB擾ăvOCɐݒ
            int yuy2Scale = ((SelectableProperty)property["YUY2XP["]).Selected;
            int yuy2Color = ((SelectableProperty)property["YUY2F"]).Selected;
            _plugin.Yuy2Scale = (Yuy2ScaleType)yuy2Scale;
            _plugin.Yuy2ColorMatrix = (Yuy2ColorMatrixType)yuy2Color;

            // 擾
            IInputInfo info = _plugin.MediaFileInfo;
            int orgWidth = info.InputImageWidth;
            int orgHeight = info.InputImageHeight;
            double rate = composition.ResolutionRate;
            int width = (int)(orgWidth * rate + 0.5);
            int height = (int)(orgHeight * rate + 0.5);
            bool resize = (width != orgWidth || height != orgHeight);

            // LbVTCYZo
            long cacheSizeOrg = orgWidth * orgHeight * 4;
            long cacheSizeRsz = width * height * 4;

            try
            {
                BeginInput();

                // ǂݍނׂt[ʒuZo
                int frame = Math.Min(
                    (int)(time * _frameRate),
                    info.InputImageFrameCount - 1);

                // ǂݎ
                using (CacheLocker bufCacheLock = new CacheLocker(cacheSizeOrg))
                {
                    byte[] buf = _plugin.ReadMediaFileImage(frame);
                    if (buf != null)
                    {
                        // 32rbgARGBrbg}bv쐬
                        using (CacheLocker bmpCacheLock = new CacheLocker(cacheSizeOrg))
                        using (Bitmap bmp = new Bitmap(orgWidth, orgHeight, PixelFormat.Format32bppArgb))
                        {
                            // sNZf[^ݒ
                            BitmapData bmpData = bmp.LockBits(
                                new Rectangle(0, 0, orgWidth, orgHeight),
                                ImageLockMode.WriteOnly,
                                PixelFormat.Format32bppArgb);
                            Marshal.Copy(buf, 0, bmpData.Scan0, (orgWidth * 4) * orgHeight);
                            bmp.UnlockBits(bmpData);
                            bmpData = null;

                            if (resize)
                            {
                                // TCY
                                using (CacheLocker canvasCacheLock = new CacheLocker(cacheSizeRsz))
                                using (Bitmap canvas = new Bitmap(width, height, PixelFormat.Format32bppArgb))
                                {
                                    using (Graphics g = Graphics.FromImage(canvas))
                                    {
                                        g.CompositingMode = CompositingMode.SourceCopy;
                                        g.CompositingQuality = CompositingQuality.HighQuality;
                                        g.InterpolationMode = InterpolationMode.NearestNeighbor;
                                        g.ScaleTransform((float)rate, (float)rate, MatrixOrder.Append);
                                        g.DrawImage(bmp, 0, 0, orgWidth, orgHeight);
                                    }

                                    CacheManager.Manager.DemandTemporalMemory(cacheSizeRsz);
                                    result = new NBitmap(canvas);
                                }
                            }
                            else
                            {
                                // TCYsv
                                CacheManager.Manager.DemandTemporalMemory(cacheSizeOrg);
                                result = new NBitmap(bmp);
                            }
                        }
                    }
                }
            }
            finally
            {
                EndInput();

                // ǂݍ߂ȂȂ΋C[W쐬
                if (result == null)
                {
                    CacheManager.Manager.DemandTemporalMemory(cacheSizeRsz);
                    result = new NBitmap(width, height);
                }

                // o̓C[WQƃLbVɒǉ
                CacheManager.Manager.AddWeakCache(result, result.DataSize);
            }

            return result;
        }

        public override byte[] GetAudio(double time, int length, IComposition composition)
        {
            // Ă΂Ȃ
            throw new NotImplementedException();
        }

        #endregion

        #region InputBase o

        public override string CorrespondingType
        {
            get { return "fBAt@C (*.*)|*.*"; }
        }

        public override string FileName
        {
            get { return (_plugin == null) ? "" : _plugin.MediaFilePath; }
        }

        public override void Initialize()
        {
            // gpȂ͂
        }

        public override bool Load(string file)
        {
            // gpȂ͂
            return false;
        }

        public override InputChild[] GetChildInput()
        {
            // gpȂ͂ꉞ
            return new InputChild[] { this };
        }

        public override object SaveInnerData()
        {
            return null;
        }

        public override void LoadInnerData(object data)
        {
            // Ȃ
        }

        #endregion

        #region PluginBase o

        public override string PluginName
        {
            get { return "AviUtl̓vOCp"; }
        }

        public override string Author
        {
            get { return "[`F"; }
        }

        public override string InfoLink
        {
            get { return "http://www.ruche-home.net/"; }
        }

        public override string Description
        {
            get
            {
                return "AviUtl̓̓vOC(*.aui)𗘗păfBAt@Cǂݍ݂܂B";
            }
        }

        #endregion

        #region IDisposable o

        public override void Dispose()
        {
            _plugin = null;
        }

        #endregion
    }
}
