﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace ruche.nive2.aviutlPluginPorts
{
    /// <summary>
    /// カレントディレクトリをスタックで保持するシングルトンクラス。
    /// </summary>
    public class CurrentDirectoryStack
    {
        /// <summary>
        /// 唯一のインスタンス。
        /// </summary>
        private static readonly CurrentDirectoryStack _instance =
            new CurrentDirectoryStack();

        /// <summary>
        /// カレントディレクトリスタック。
        /// </summary>
        private Stack<string> _curDirStack = new Stack<string>();

        /// <summary>
        /// 唯一のインスタンスを取得する。
        /// </summary>
        public static CurrentDirectoryStack Instance
        {
            get { return _instance; }
        }

        /// <summary>
        /// コンストラクタ。
        /// </summary>
        private CurrentDirectoryStack()
        {
        }

        /// <summary>
        /// カレントディレクトリを変更してスタックにプッシュする。
        /// </summary>
        /// <param name="dirPath">ディレクトリパス。</param>
        /// <returns>成功したならば true 、そうでなければ false 。</returns>
        public bool Push(string dirPath)
        {
            try
            {
                _curDirStack.Push(dirPath);
                Directory.SetCurrentDirectory(dirPath);
            }
            catch
            {
                return false;
            }

            return true;
        }

        /// <summary>
        /// カレントディレクトリを元に戻し、スタックからポップする。
        /// </summary>
        /// <returns>成功したならば true 、そうでなければ false 。</returns>
        public bool Pop()
        {
            try
            {
                Directory.SetCurrentDirectory(_curDirStack.Pop());
            }
            catch
            {
                return false;
            }

            return true;
        }
    }
}
